# - Check if the python module exists.
# CHECK_PYTHON_MODULES(MODULE VARIABLE)
# - macro which checks the python module exists.
#  MODULE  - python module
#  VARIABLE - variable to return result

include(ParseArguments)

MACRO(CHECK_PYTHON_MODULE MODULE VARIABLE)
  PARSE_ARGUMENTS(MODULE "" "REQUIRED" ${ARGN})
  
  IF("${VARIABLE}" MATCHES "^${VARIABLE}$")
    MESSAGE(STATUS "Looking for ${MODULE} in python modules")

    FIND_PACKAGE(PythonInterp REQUIRED)
    IF (PYTHONINTERP_FOUND)
	EXECUTE_PROCESS(
	    COMMAND ${PYTHON_EXECUTABLE} -c "import ${MODULE}" 
	    RESULT_VARIABLE RESULT
	    OUTPUT_QUIET
	    ERROR_QUIET
	)
    ELSE (PYTHONINTERP_FOUND)
	SET (RESULT 1)
    ENDIF (PYTHONINTERP_FOUND)
    
    IF (RESULT)
	MESSAGE(STATUS "Looking for ${MODULE} in python modules - not found")
	SET(${VARIABLE} "" CACHE INTERNAL "Have python module ${MODULE}")
	IF (MODULE_REQUIRED)
	    MESSAGE(FATAL_ERROR "Required python module '${MODULE}' is not found")
	ENDIF (MODULE_REQUIRED)
    ELSE (RESULT)    
	MESSAGE(STATUS "Looking for ${MODULE} in python modules - found")
	SET(${VARIABLE} 1 CACHE INTERNAL "Have python module ${MODULE}")
    ENDIF(RESULT)    
  ENDIF("${VARIABLE}" MATCHES "^${VARIABLE}$")
ENDMACRO(CHECK_PYTHON_MODULE)