#include <stdio.h>
#include <stdlib.h>

#ifdef DICT_HW_MEASURE_TIMINGS
# include <sys/time.h>
#endif /* DICT_HW_MEASURE_TIMINGS */

#include "normxcorr_hw.h"
#include "normxcorr_hw_msg.h"

#include "helpers.h"

#include "dict_image.h"

static int dictImageFreeBuffer(DICTContext ctx, DICTImageType image_type, void *image_buf) {
    switch (image_type) {
#ifdef DICT_TIFF_SUPPORT
        case DICT_IMAGE_TIFF:
	    _TIFFfree(image_buf);
	break;
#endif /* DICT_TIFF_SUPPORT */
    }
    
    return 0;
}

int dictImageFree(DICTContext ps) {
    if (ps->image_buf) {
	dictImageFreeBuffer(ps, ps->image_type, ps->image_buf);
	ps->image_buf = NULL;
    }
    if (ps->image2) {
	free(ps->image2);
	ps->image2 = NULL;
    }
    if (ps->image) {
	free(ps->image);
	ps->image = NULL;
    }
    
    return 0;
}

#ifdef USE_SSE
    // needs extra 32 bytes in the end of array, this is do20
static inline void reduct_ssse3(unsigned char *dst, unsigned char *src, int size) {
    int i;
    volatile int j;		// to prevent reduction of the loop
    unsigned char *end;

    ALGNW char shuf[64] ALGNL;
    char *shuf1 = shuf;
    char *shuf2 = shuf+16;
    char *shuf3 = shuf+32;
    uint16_t *mul = (uint16_t*)(shuf+48);

    memset(shuf, 0xFF, 48);
    for (i=0;i<16;i+=4) {
	shuf1[i] = i;
	shuf1[i+2] = i + 1;
    }
    for (i=0;i<8;i+=2) {
	shuf2[i] = 2 * i + 2;
	
	shuf3[i + 8] = 2 * i + 2;
    }
    for (i=0;i<8;i++) {
	mul[i] = 0x2AAA;
    }

    long aligned_size = calc_alloc(size, 32);
    long val = -aligned_size;
    unsigned char *src2 = src + 4 * aligned_size;
    unsigned char *dst2 = dst + aligned_size;

	__asm__ __volatile__ (
            "prefetchnta (%1)				\n\t"
            "push 	%2				\n\t"
	    "movdqa	(%3), %%xmm15			\n\t"
	    "movdqa	16(%3), %%xmm14			\n\t"
	    "movdqa	32(%3), %%xmm13			\n\t"
	    "movdqa	48(%3), %%xmm12			\n\t"

            "movdqa	(%1,%2,4), %%xmm0		\n\t"
            "movdqa	16(%1,%2,4), %%xmm1		\n\t"
        "1:						\n\t"
	    "prefetchnta 64(%1)				\n\t"
	    
	    "movdqa	%%xmm0, %%xmm5			\n\t"
	    "pshufb	%%xmm15, %%xmm0			\n\t"

	    "movdqa	%%xmm1, %%xmm6			\n\t"
	    "pshufb	%%xmm15, %%xmm1			\n\t"

            "movdqa	32(%1,%2,4), %%xmm2		\n\t"
            "movdqa	48(%1,%2,4), %%xmm3		\n\t"

	    "pshufb	%%xmm14, %%xmm5			\n\t"
	    "pshufb	%%xmm13, %%xmm6			\n\t"
	    "por	%%xmm6, %%xmm5			\n\t"


	    "phaddw	%%xmm1, %%xmm0			\n\t"
	    "paddw	%%xmm5, %%xmm0			\n\t"
	    "pmulhrsw	%%xmm12, %%xmm0			\n\t"
//------
	    
	    "movdqa	%%xmm2, %%xmm5			\n\t"
	    "pshufb	%%xmm15, %%xmm2			\n\t"
	    "movdqa	%%xmm3, %%xmm6			\n\t"
	    "pshufb	%%xmm15, %%xmm3			\n\t"

            "movdqa	64(%1,%2,4), %%xmm7		\n\t"
            "movdqa	80(%1,%2,4), %%xmm8		\n\t"

	    "pshufb	%%xmm14, %%xmm5			\n\t"
	    "pshufb	%%xmm13, %%xmm6			\n\t"
	    "por	%%xmm6, %%xmm5			\n\t"

	    "phaddw	%%xmm3, %%xmm2			\n\t"
	    "paddw	%%xmm5, %%xmm2			\n\t"
	    "pmulhrsw	%%xmm12, %%xmm2			\n\t"

//-----
	    "packuswb	%%xmm2, %%xmm0			\n\t"
            "movntps	%%xmm0, (%0,%2)			\n\t"

            "prefetchnta 128(%1)			\n\t"
	    
	    "movdqa	%%xmm7, %%xmm5			\n\t"
	    "pshufb	%%xmm15, %%xmm7			\n\t"
	    "movdqa	%%xmm8, %%xmm6			\n\t"
	    "pshufb	%%xmm15, %%xmm8			\n\t"

            "movdqa	96(%1,%2,4), %%xmm9		\n\t"
            "movdqa	112(%1,%2,4), %%xmm10		\n\t"

	    "pshufb	%%xmm14, %%xmm5			\n\t"
	    "pshufb	%%xmm13, %%xmm6			\n\t"
	    "por	%%xmm6, %%xmm5			\n\t"

	    "phaddw	%%xmm8, %%xmm7			\n\t"
	    "paddw	%%xmm5, %%xmm7			\n\t"
	    "pmulhrsw	%%xmm12, %%xmm7			\n\t"
//------
	    
	    "movdqa	%%xmm9, %%xmm5			\n\t"
	    "pshufb	%%xmm15, %%xmm9			\n\t"
	    "movdqa	%%xmm10, %%xmm6			\n\t"
	    "pshufb	%%xmm15, %%xmm10		\n\t"

	    "pshufb	%%xmm14, %%xmm5			\n\t"
	    "pshufb	%%xmm13, %%xmm6			\n\t"
	    "por	%%xmm6, %%xmm5			\n\t"

            "movdqa	128(%1,%2,4), %%xmm0		\n\t"
            "movdqa	144(%1,%2,4), %%xmm1		\n\t"

	    "phaddw	%%xmm10, %%xmm9			\n\t"
	    "paddw	%%xmm5, %%xmm9			\n\t"
	    "pmulhrsw	%%xmm12, %%xmm9			\n\t"
//-----

	    "packuswb	%%xmm9, %%xmm7			\n\t"
            "movntps	%%xmm7, 16(%0,%2)		\n\t"

            "add	$32, %2				\n\t"
            "jnz 	1b				\n\t"
            "pop 	%2				\n\t"

    :
    : "r" (dst2), "r" (src2), "r" (val), "r" (&shuf)
    );
}
#endif /*USE_SSE */


int dictReduceImage(DICTContext ps, unsigned char *res, void *img, int width, int height) {
    int size = width * height;
//    unsigned char *res = ps->image;

#ifdef DICT_TIFF_SUPPORT
    unsigned char *image = (unsigned char*)img;
#endif /* DICT_TIFF_SUPPORT */

    switch (ps->image_type) {
#ifdef DICT_TIFF_SUPPORT
        case DICT_IMAGE_TIFF:
	    if (ps->matlab_mode) {
/*
		for (int i = 0; i < size; ++i) {
		    int col = i % height;
		    int lin = i / height;
		    res[i] = rintf(((float)(image[pos] + image[pos+1] + image[pos+2]))/3);
		}
*/
/*
    		for (int col = 0; col < height; col++) {
		    for (int lin = 0; lin < width; lin++) {
			int i = lin*height + col;
			int pos = 4*(col * width + lin);
			unsigned short sum = 85 * (image[pos] + image[pos+1] + image[pos+2]);
	        	res[i] = (90 + sum + (sum>>8)) >> 8;
		    }
		}
*/

		    // We doing that to benefit from CPU caches		
		int aligned_width = (width / TRANSPOSE_SIZE) * TRANSPOSE_SIZE;
		int aligned_height = (height / TRANSPOSE_SIZE) * TRANSPOSE_SIZE;

    		for (int col = 0; col < aligned_height; col+=TRANSPOSE_SIZE) {
		    for (int lin = 0; lin < aligned_width; lin+=TRANSPOSE_SIZE) {

    			for (int col2 = 0; col2 < TRANSPOSE_SIZE; col2++) {
			    for (int lin2 = 0; lin2 < TRANSPOSE_SIZE; lin2++) {
	    			int i = (lin+lin2)*height + (col+col2);
				int pos = 4*((col+col2) * width + (lin+lin2));
				
				unsigned short sum = 85 * (image[pos] + image[pos+1] + image[pos+2]);
		        	res[i] = (90 + sum + (sum>>8)) >> 8;
			    }
			}
		    }
		}
		
    		for (int col = aligned_height; col < height; col++) {
		    for (int lin = aligned_width; lin < width; lin++) {
			int i = lin*height + col;
			int pos = 4*(col * width + lin);
			unsigned short sum = 85 * (image[pos] + image[pos+1] + image[pos+2]);
	        	res[i] = (90 + sum + (sum>>8)) >> 8;
		    }
		}
	    } else {
#ifdef USE_SSE
		reduct_ssse3(res, image, size);
#else /* USE_SSE */
	        for (int i = 0; i < size; ++i) {
		    int pos = 4 * i;
		    unsigned short sum = 85 * (image[pos] + image[pos+1] + image[pos+2]);
	            res[i] = (90 + sum + (sum>>8)) >> 8;
//		    res[i] = rintf(((double)(image[pos] + image[pos+1] + image[pos+2]))/3);
		}
#endif /* USE_SSE */
	    }
	break;
#endif /* DICT_TIFF_SUPPORT */
    }
    
    return 0;
}

int dictImageLoadTemplateImage(DICTContext ps, const char *name) {
#ifdef DICT_TIFF_SUPPORT
    int err;
    uint32_t w, h;

#ifdef DICT_HW_MEASURE_TIMINGS
    int time;
    struct timeval tv1, tv2, tv3;
    gettimeofday(&tv1, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
    
    TIFF* tif = TIFFOpen(name, "r");
    if (!tif) return DICT_ERROR_IMAGE;
    
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
    
    if (ps->image_buf) {
	dictImageFreeBuffer(ps, ps->image_type, ps->image_buf);
    }
    
    if (ps->image) {
	free(ps->image);
    }
    
    if (ps->image2) {
	free(ps->image2);
    }

    ps->image_type = DICT_IMAGE_TIFF;
	// we need extra 32 bytes in the end for do20 sse code
    int alloc_size = 32 + calc_alloc(w * h, 32);
    ps->image_buf = _TIFFmalloc(alloc_size * sizeof (uint32_t));
    ps->image = (unsigned char*)malloc(alloc_size * sizeof(unsigned char));
    ps->image2 = (unsigned char*)malloc(alloc_size * sizeof(unsigned char));

    if ((!ps->image)||(!ps->image2)||(!ps->image_buf)) {
	reportError("Problem allocating memory for image %ix%i", w, h);
	return DICT_ERROR_MALLOC;
    }

    err = dictSetDimensions(ps, w, h);    
    if (err) return err;

    if (TIFFReadRGBAImage(tif, w, h, (uint32_t*)ps->image_buf, 0)) {
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv3, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
	dictReduceImage(ps, ps->image, ps->image_buf, w, h);
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv2, NULL);
	time = (tv2.tv_sec - tv3.tv_sec) * 1000000 + (tv2.tv_usec - tv3.tv_usec);
	ps->time[13] += time;
#endif /* DICT_HW_MEASURE_TIMINGS */

    }
    
    TIFFClose(tif);

#ifdef DICT_HW_MEASURE_TIMINGS
    gettimeofday(&tv2, NULL);
    ps->time[12] += (tv2.tv_sec - tv1.tv_sec) * 1000000 + (tv2.tv_usec - tv1.tv_usec) - time;
#endif /* DICT_HW_MEASURE_TIMINGS */

    return 0;
#else /* DICT_TIFF_SUPPORT */
    return DICT_ERROR_IMAGE;
#endif /* DICT_TIFF_SUPPORT */
}

int dictImageLoadImage(DICTContext ps, const char *name, unsigned char *buf) {
#ifdef DICT_TIFF_SUPPORT
#ifdef DICT_HW_MEASURE_TIMINGS
    int time;
    struct timeval tv1, tv2, tv3;
    gettimeofday(&tv1, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */

    TIFF* tif = TIFFOpen(name, "r");
    if (!tif) return DICT_ERROR_IMAGE;

    if (TIFFReadRGBAImage(tif, ps->width, ps->height, (uint32_t*)ps->image_buf, 0)) {
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv3, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
	dictReduceImage(ps, buf?buf:ps->image, ps->image_buf, ps->width, ps->height);
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv2, NULL);
	time = (tv2.tv_sec - tv3.tv_sec) * 1000000 + (tv2.tv_usec - tv3.tv_usec);
	ps->time[15] += time;
#endif /* DICT_HW_MEASURE_TIMINGS */
    }

    TIFFClose(tif);

#ifdef DICT_HW_MEASURE_TIMINGS
    gettimeofday(&tv2, NULL);
    ps->time[14] += (tv2.tv_sec - tv1.tv_sec) * 1000000 + (tv2.tv_usec - tv1.tv_usec) - time;
#endif /* DICT_HW_MEASURE_TIMINGS */
    return 0;
#else /* DICT_TIFF_SUPPORT */
    return DICT_ERROR_IMAGE;
#endif /* DICT_TIFF_SUPPORT */
}
